import socket
import json
import uuid
from pycoze import utils


params = utils.params
tcp_port = params["tcpPort"]


class TcpSocket:
    def __init__(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.connect(("localhost", int(tcp_port)))
        self.is_connected = True

    def post(self, subject, message):
        subject = "python-event:" + subject
        message = message.copy()
        message["subject"] = subject
        message = json.dumps(message)
        message_bytes = message.encode()
        message_length = len(message_bytes)
        message_length_bytes = message_length.to_bytes(4, byteorder='big')

        self.socket.sendall(message_length_bytes + message_bytes)

    def recv(self):
        # 接收消息长度
        message_length_bytes = self.socket.recv(4)
        message_length = int.from_bytes(message_length_bytes, byteorder='big')
        # 初始化接收到的消息
        message = b''
        # 循环接收直到收到完整的消息
        while len(message) < message_length:
            # 接收剩余的消息
            chunk = self.socket.recv(message_length - len(message))
            if not chunk:  # 如果没有接收到数据，则断开连接
                raise RuntimeError("Socket connection broken")
            message += chunk
        message = json.loads(message.decode())
        return message

    def post_and_recv_result(self, subject, message):
        message = message.copy()
        message["uid"] = str(uuid.uuid4())
        socket_subscribe(subject + ":" +  message["uid"]) # 先订阅,再发送,防止消息丢失
        self.post(subject, message)
        ret_val = self.recv()        
        result = ret_val["result"] if "result" in ret_val else None # 传输时，可能会丢失result字段，因此需要判断
        return result

    def close(self):
        if self.is_connected:
            self.is_connected = False

    def __del__(self):
        # 垃圾回收时自动关闭socket
        if self.is_connected:
            self.close()

    def __exit__(self, exc_type, exc_value, traceback):
        # with 语句块结束时自动触发的
        if self.is_connected:
            self.close()


socket = TcpSocket()

def socket_subscribe(subject: str):
    subject = "python-event:" + subject
    socket.post(subject, {"subscribe": subject})
    return socket


import re
from .chat import yes_or_no, extract


def contains_chinese(text):
    pattern = re.compile(r'[\u4e00-\u9fa5]')  # Regular expression to match Chinese characters
    return bool(re.search(pattern, text))


requirement = """
Please reflect the scene content as prompts for the drawing AI.

## Prompt Concept
- A prompt is used to describe the image content, composed of common words, using English commas (",") as separators. For example, a prompt like "woman" indicates that the image should include a woman.

## Tag Restrictions
- Tags should be described using English words or phrases, avoiding Chinese.
- Tags can only contain keywords or key phrases, and should not include personal names, place names, etc.
- Tags should try to preserve physical characteristics of people, like body shape and hairstyle, but not use personal names, instead using terms like "man" to refer to people.
- The number of tags in a prompt is limited to 40, and the number of words is limited to 60.

## Incorrect Examples of Prompts
"In the bustling Shanghai Bund, there is a young man named Li Yang."
The prompt includes non-keywords like "there is", as well as the personal name "Li Yang" and the place name "Shanghai."
It should be modified to "a young man, in the bustling Bund"

"""

# As the LLM used is not very effective, negative prompts are not suitable for generation by LLM at this stage
# - Negative prompts describe content that should not appear in the image, for example if "bird, man" appears in the negative prompt, it means the image should not include "birds and men".


def text_to_image_prompt(query, style, negative_style, with_prompt="best quality,4k,", with_negative_prompt=""):
    query = query.replace("{", "【").replace("}", "】")
    needed = f"The scene I need: {query}"
    style = style.replace(",", " ").replace("，", " ")
    if len(style) > 0:
        needed += f"\nThe style I need: {style}"
    # if len(negative_style) > 0:
    #     needed += f"\nStyles to avoid: {negative_style}"

    for i in range(15):
        try:
            print("doing")
            output_obj = extract([("prompt", "Describe the image content with keywords")], needed + "\n" + requirement)
            print("done")
            if contains_chinese(output_obj["prompt"]):
                print("Contains Chinese, regenerating")
                continue
            if yes_or_no("Does it include personal names:\n"+output_obj["prompt"]):
                print(output_obj["prompt"]+" contains personal names, correcting")
                output_obj["prompt"] = extract(
                    [("text without personal names", "Modified result (personal names can be changed to man, woman, he, she, etc.)")],
                    "Modify the following text, personal names can be changed to man, woman, he, she, etc.: \n"+output_obj["prompt"])["text without personal names"]
            break
        except Exception as e:
            print(e)
    output_obj["prompt"] = with_prompt + output_obj["prompt"].replace(" and ", ",")
    output_obj["negative_prompt"] = with_negative_prompt
    output_obj["with_prompt"] = with_prompt
    output_obj["with_negative_prompt"] = with_negative_prompt
    output_obj["query"] = query
    output_obj["style"] = style
    output_obj["negative_style"] = negative_style
    return output_obj



from .lib.window_cls import WindowCls

class Api:
    def __init__(self) -> None:
        self.window = WindowCls()

oper = Api()

# from ps_view import ViewCls, WebsiteViewCls, FileViewCls, DirectoryViewCls, WorkflowCls